@@ -1,5 +1,4 @@
 
-
 variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, dataset=NULL) {
 
   ### Checking
@@ -8,6 +7,7 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, data
       if (min(Y %in% names(dataset))!=1) stop("'Y' does not exist in 'dataset'!")
       if (min(Y %in% names(dataset))==1) Y <- dataset[, Y, with=FALSE] 
 
+
       if(!is.null(H)) {
           if (min(H %in% names(dataset))!=1) stop("'H' does not exist in 'dataset'!")
           if (min(H %in% names(dataset))==1) H <- dataset[, H, with=FALSE] }
@@ -15,15 +15,18 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, data
           if (min(H2 %in% names(dataset))!=1) stop("'H2' does not exist in 'dataset'!")
           if (min(H2 %in% names(dataset))==1) H2 <- dataset[, H2, with=FALSE] }
 
+
       if(!is.null(w_final)) {
           if (min(w_final %in% names(dataset))!=1) stop("'w_final' does not exist in 'dataset'!")
           if (min(w_final %in% names(dataset))==1) w_final <- dataset[, w_final, with=FALSE] }
 
+
        if (!is.null(period)) {
             if (min(period %in% names(dataset))!=1) stop("'period' does not exist in 'dataset'!")
             if (min(period %in% names(dataset))==1) period <- dataset[, period, with=FALSE]}
       }
 
+
   # Y
   Y <- data.table(Y, check.names=TRUE)
   n <- nrow(Y)
@@ -40,6 +43,7 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, data
   if (is.null(names(H))) stop("'H' must be colnames")
   H[, (names(H)):=lapply(.SD, as.character)]
 
+
   # H2
   H2 <- data.table(H2)
   if (nrow(H2) != n) stop("'H2' length must be equal with 'Y' row count")
@@ -48,6 +52,7 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, data
   if (is.null(names(H2))) stop("'H2' must be colnames")
   H2[, (names(H2)):=lapply(.SD, as.character)]
 
+
   # w_final
   w_final <- data.frame(w_final)
   if (nrow(w_final) != n) stop("'w_final' must be equal with 'Y' row count")
@@ -56,6 +61,7 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, data
   if (!is.numeric(w_final)) stop("'w_final' must be numerical")
   if (any(is.na(w_final))) stop("'w_final' has unknown values")
 
+
   # period     
   if (!is.null(period)) {
        period <- data.table(period)
@@ -93,6 +99,7 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, data
     N_h <- Nh[, list(N_h = sum(w_final, na.rm = F)), Nh[, c(1:(1+np)), with=FALSE]]
   }
 
+
   # N_h2
   if (!is.null(N_h2)) {
       N_h2 <- data.table(N_h2)
@@ -114,12 +121,16 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, data
     setkeyv(N_h2, names(N_h2)[c(1:(1+np))])
   } else stop ("N_h2 is not defined!")
 
+
   if (all(names(H)==names(H2))) {
       if (!is.null(N_h2))  setnames(N_h2, names(N_h2), paste0(names(N_h2),"2"))  
      setnames(H2, names(H2), paste0(names(H), "2"))  }
 
 
 
+
+
+
   ### Calculation
   
   # z_hi
@@ -127,16 +138,21 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, data
   Ys <- copy(Y)
   Ys[, paste0(names(Y),"_sa") := lapply(Y, function(x) w_final * x^2)]
   Ys[, paste0(names(Y),"_sb") := lapply(Y, function(x) x * w_final)]
+  Ys[, paste0(names(Y),"_sc") := lapply(Y, function(x) x ^ 2)]
+  Ys[, paste0(names(Y),"_sd") := Y]
+
 
   Ys <- data.table(H, H2, Ys)
   if (!is.null(period)) Ys <- data.table(period, Ys)
 
+
   # n_h1
   n_h1 <- data.table(H)
   if (!is.null(period))   n_h1 <- data.table(period, n_h1)
   n_h1 <- n_h1[,.N, keyby=c(names(n_h1))]
   setnames(n_h1,"N","n_h1")
 
+
   if (any(n_h1$n_h1 == 1)) {
     print("There is stratas, where n_h1 == 1")
     print("Not possible to estimate the variance in these stratas!")
@@ -145,6 +161,7 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, data
     print(data.frame(n_h1)[nh1,])
   }
 
+
   # n_h2
   n_h2 <- data.table(H2)
   if (!is.null(period)) n_h2 <- data.table(period, n_h2)
@@ -152,6 +169,7 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, data
   n_h2 <- n_h2[,.N, keyby=nn_h2]
   setnames(n_h2,"N","n_h2")
 
+
   if (any(n_h2$n_h2 == 1)) {
     print("There is stratas, where n_h2 == 1")
     print("Not possible to estimate the variance in these stratas!")
@@ -160,26 +178,47 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, data
     print(data.frame(n_h2)[nh2,])
   }
 
+
   z_h_h2 <- Ys[, lapply(.SD, sum, na.rm=TRUE), keyby = c(names(Ys)[1:(2+np)]),
                       .SDcols = names(Ys)[-(0:(ncol(Y)+2+np))]]
 
+
   setkeyv(z_h_h2, names(z_h_h2)[c(1:(1+np))]) 
 
+
   z_h_h2 <- merge(z_h_h2, n_h1)
   z_h_h2 <- merge(z_h_h2, N_h)
 
+
   pop <- z_h_h2[[names(N_h)[ncol(N_h)]]]
 
+
+  z_h_h2[, paste0(names(Y),"_sc") := lapply(.SD[, 
+           paste0(names(Y),"_sc"), with=FALSE], function(x)
+           x * pop ^ 2 * ( 1/n_h1 - 1/pop)/(n_h1-1))]
+
+
+  z_h_h2[, paste0(names(Y),"_sd") := lapply(.SD[,
+           paste0(names(Y),"_sd"), with=FALSE], function(x) 
+                 (1/n_h1) * x ^ 2  * pop ^ 2 * (1/n_h1 - 1/pop)/(n_h1-1))]
+
+
+  z_h_h2[n_h1==1, paste0(names(Y),"_sc"):=NA]
+  z_h_h2[n_h1==1, paste0(names(Y),"_sd"):=NA]
+
+
   nameszh2 <- names(H2)
   if (!is.null(period)) nameszh2 <- c(names(period), nameszh2)
   
   zh2 <- z_h_h2[, lapply(.SD, sum, na.rm=TRUE), keyby = nameszh2,
                       .SDcols = names(z_h_h2)[-(1:(2+np))]] 
 
+
   F_h2 <- merge(N_h2, n_h2)
   f_h2 <- F_h2[, np+3, with=F] / F_h2[, np+2, with=FALSE]
   setnames(f_h2, "n_h2", "f_h2")
 
+
   if (any(f_h2 > 1)) {    
     print("There is stratas, where f_h2 > 1")
     print("At these stratas estimation of variance will be 0")
@@ -187,26 +226,35 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, data
     f_h2[f_h2 > 1] <- 1
   }
 
+
   zh2 <- merge(zh2, F_h2)
   pop2 <- zh2[[names(N_h2)[ncol(N_h2)]]]
   nh2 <- zh2[[names(n_h2)[ncol(n_h2)]]]
 
+
   # s2
-  s2_g <- zh2[,mapply(function(sa, sb) sa/(pop2-1)-pop2/(pop2-1)*(sb/pop2)^2,
+  s2_g <- zh2[,mapply(function(sa, sb, sc, sd) sa/(pop2-1)-pop2/(pop2-1)*((sb/pop2)^2-(sc-sd)/pop2^2),
               zh2[, paste0(names(Y),"_sa"), with=FALSE], 
-              zh2[, paste0(names(Y),"_sb"), with=FALSE])]
+              zh2[, paste0(names(Y),"_sb"), with=FALSE],
+              zh2[, paste0(names(Y),"_sc"), with=FALSE],
+              zh2[, paste0(names(Y),"_sd"), with=FALSE])]
+
 
   # var_g 
-  if (nrow(s2_g)==1) s2_g <- t(s2_g)
+  if (is.null(nrow(s2_g))) s2_g <- t(s2_g)
   s2_g <- data.table(s2_g)
   setnames(s2_g, names(s2_g), names(Y))
 
+
   s2g <- data.table(zh2[, nn_h2, with=FALSE], s2_g)
 
+
   s2_g <- pop2^2*(1/nh2-1/pop2) * s2_g
 
+
   if (np>0) s2_g <- data.table(zh2[, names(period), with=FALSE], s2_g)
 
+
   # Variance_est
   if (np==0) {var_est <- data.table(t(colSums(s2_g, na.rm=TRUE)))
            } else var_est <- s2_g[, lapply(.SD, sum, na.rm=TRUE), 
@@ -214,4 +262,4 @@ variance_othstr <- function(Y, H, H2, w_final, N_h=NULL, N_h2, period=NULL, data
                                       .SDcols = names(Y)]
   list(s2g=s2g,
        var_est=var_est)
-}
+}
\ No newline at end of file
